"""."""

import cv2

from kinematic_tracker.association.metric_giou_aligned import MetricGIoUAligned


def test_empty_det(filters: list[cv2.KalmanFilter], driver: MetricGIoUAligned) -> None:
    """."""
    driver = MetricGIoUAligned(100, 500, 6, (0, 1, 2, 3, 4, 5))
    metric = driver.compute_metric([], filters)
    assert metric.shape == (0, 3)
